searchData={"items":[{"type":"module","title":"Jcs","doc":"A pure Elixir implementation of RFC 8785: JSON Canonicalization Scheme (JCS)\n\nBased on Python 3 implementation at https://github.com/titusz/jcs\n\nRequires Erlang OTP 25 (Ryu `float_to_binary(number, [:short])` support), and\ntherefore Elixir 1.14.","ref":"Jcs.html"},{"type":"function","title":"Jcs.encode/1","doc":"Encodes data into a JSON string, in a canonical form according to\n[RFC 8785](https://www.rfc-editor.org/rfc/rfc8785#name-generation-of-canonical-jso).\n\nCanonicalizes nested object entries and sorts them by their names.\n\nEntries with the same name are sorted by value.\n\nNumbers are encoded to produce the shortest exact values possible,\nusing the Erlang function `:erlang.float_to_binary/2`, which seems\nto have differing results depending on the OTP release. Steps\nfrom [ECMA-262 - Abstract Operations - 7.1.12.1 - NumberToString](https://262.ecma-international.org/10.0/index.html#sec-abstract-operations)\nare applied to the results of the conversion to produce the final\nencoding.\n\nThe ordering for the name and value sorting is determined by converting\neach name and value into UTF-16, while the actual resultant JSON string is\nencoded in UTF-8, according to the particular rules in\n[RFC 8785 - 3.2.2.2 - Serialization of Strings](https://www.rfc-editor.org/rfc/rfc8785#name-serialization-of-strings).","ref":"Jcs.html#encode/1"},{"type":"function","title":"Jcs.encode_basestring/1","doc":"Returns a JSON representation of a string, escaping characters\nper RFC 8785:\n\nIf the Unicode value falls within the traditional ASCII control character\nrange (U+0000 through U+001F), it MUST be serialized using lowercase\n4-digit hexadecimal Unicode notation (\"\\uhhhh\") unless it is in\nthe set of predefined JSON control characters U+0008, U+0009, U+000A,\nU+000C, or U+000D, which MUST be serialized as \"\b\", \"\t\", \"\n\", \"\f\",\nand \"\r\", respectively.\n\nIf the Unicode value is outside of the ASCII control character range, it\nMUST be serialized \"as is\" unless it is equivalent to U+005C () or\nU+0022 (\"), which MUST be serialized as \"\\\" and \"\"\", respectively.","ref":"Jcs.html#encode_basestring/1"},{"type":"function","title":"Jcs.encode_basestring_ascii/1","doc":"Returns a JSON representation of a string, escaping characters\nin the range U+0000 through U+007E per RFC 8785. Values above\nU+0073 are serialized using lowercase hexadecimal Unicode notation\n(\"\\uhhhh\" or \"\\u{hhhhh}\")\n\nSee `encode_basestring/1` for details of the encoding of values\nU+005C (), U+0022 (\") and U+0000 through U+001F.","ref":"Jcs.html#encode_basestring_ascii/1"},{"type":"function","title":"Jcs.escape_unicode/1","doc":"Given either a String, or single codepoint, returns a string\nconcatenating  \"\\uhhhh\" and \"\\u{hhhhh}\" elements.","ref":"Jcs.html#escape_unicode/1"},{"type":"function","title":"Jcs.to_utf16/1","doc":"Given either a binary, a list of codepoints,\na single codepoint (unicode character) or its integer value,\nreturns a flattened list of UTF-16 encoded values. This list\ncan be used to sort object names as specified in the RFC.","ref":"Jcs.html#to_utf16/1"},{"type":"extras","title":"JCS","doc":"# Jcs\n\nA pure Elixir implementation of \n[RFC 8785: JSON Canonicalization Scheme (JCS)](https://www.rfc-editor.org/rfc/rfc8785).\n\nJCS can be used to establish a canonical deterministic representation \nof linked JSON data. These represenations can then be used in \nestablishing identity proofs. \n\nFor an example, see the [W3C Data Integrity 1.0 report](https://www.w3.org/community/reports/credentials/CG-FINAL-data-integrity-20220722/#proofs).\n\nThat report also gives as an example usage identity proofs, the ability to \nauthenticate as an entity identified by a [Decentralized Identifier (DID)](https://www.w3.org/TR/did-core/).\n\nThe JSON encoding here is probably orders of magnitude slower than the Jason \nlibrary. There is no attempt here to decent better memory management in \nbuilding the output, and sorting object properties based on their\nUTF-16-encoded keys can probably be greatly improved also.\n\nPull requests are gratefully encouraged!\n\nCode is based on the [Python 3 jcs package](https://github.com/titusz/jcs).\n\nOther language implementations are listed [here](https://github.com/cyberphone/json-canonicalization).\n\nTest suites are from:\n  * [JSON-LD 1.1 Processing Algorithms and API - Test Suite](https://w3c.github.io/json-ld-api/tests/)\n  * [cyberphone/json-canonicalization - On-line Browser Test](https://cyberphone.github.io/doc/security/browser-json-canonicalization.html)\n  * [Java implementation by Samuel Erdtman](https://github.com/erdtman/java-json-canonicalization)\n  * [RFC 8785 - Appendix B. Number Serialization Samples](https://www.rfc-editor.org/rfc/rfc8785#name-number-serialization-sample).","ref":"readme.html"},{"type":"extras","title":"Installation - JCS","doc":"If [available in Hex](https://hex.pm/docs/publish), the package can be installed\nby adding `jcs` to your list of dependencies in `mix.exs`:\n\n```elixir\ndef deps do\n  [\n    {:jcs, \"~> 0.2.0\"}\n  ]\nend\n```","ref":"readme.html#installation"},{"type":"extras","title":"Usage - JCS","doc":"```elixir\nJcs.encode(%{\"aa\" => 200, \"b\" => 100.0, \"西葛西駅\" => [200, \"station\"], \"a\" => \"hello\\tworld!\"})\n  \n\"{\\\"a\\\":\\\"hello\\\\tworld!\\\",\\\"aa\\\":200,\\\"b\\\":100,\\\"西葛西駅\\\":[200,\\\"station\\\"]}\"\n```\n\nDocumentation can be generated with [ExDoc](https://github.com/elixir-lang/ex_doc)\nand published on [HexDocs](https://hexdocs.pm). Once published, the docs can\nbe found at  .","ref":"readme.html#usage"},{"type":"extras","title":"Notes on encoding floating point values - JCS","doc":"This library depends on using the Ryu algorithm that was added to Erlang\nin OTP 25, via the `:erlang.float_to_binary/2` function and the new `:short`\noption. See https://www.erlang.org/blog/my-otp-25-highlights/#new-option-short-for-erlangfloat_to_list2-and-erlangfloat_to_binary2 for \nmore information.\n\nAlso note that while the RFC states that the Ryu algorithm is compliant for \nencoding floats, the Erlang implementation encodes integral values like 1e+23 \nas \"1.0e23\", where as RFC 8785 would encode this as \"1e+23\". \n\nWhen attempting to handle all the test cases in the RFC appendix, it appears\nas though not all of the IEEE754 double value examples are able to be encoded \ninto Elixir floats. These are commented out as \"Elixir can not set this value\"\nin the JcsNumbersTest test module.","ref":"readme.html#notes-on-encoding-floating-point-values"}],"content_type":"text/markdown"}